#include   <stdio.h>
#include   <stdlib.h>
#include   <visa.h>
#include   "age5270.h"
 
void check_err (ViSession vi, ViStatus ret) {
 ViInt32    	inst_err;
 ViChar		    err_msg[256];
 if(VI_SUCCESS > ret) {
     if ( age5270_INSTR_ERROR_DETECTED == ret ) {
         age5270_error_query(vi, &inst_err, err_msg);
         printf("Instrument Error: %ld\n %s\n", inst_err, err_msg);
	 }
     else {
         age5270_error_message(vi, ret, err_msg);
         printf("Driver Error: %ld\n %s\n", ret, err_msg);
	 }
 }
}

void perform_meas (ViSession vi, ViStatus ret) /* 1 */
{
ViInt32 drain; /* 3 */
ViInt32 gate;
ViInt32 source;
ViInt32 bulk;
drain = 1; /* SMU1 */
gate = 2; /* SMU2 */
source = 4; /* SMU4 */
bulk = 6; /* SMU6 */ /* 11 */
ret = age5270_setSwitch(vi, drain, 1); /* 13 */
ret = age5270_setSwitch(vi, gate, 1);
ret = age5270_setSwitch(vi, source, 1);
ret = age5270_setSwitch(vi, bulk, 1);
check_err (vi, ret); /* 17 */
ViReal64 vd; /* 19 */
ViReal64 vg;
ViReal64 idcomp;
ViReal64 igcomp;
ViReal64 meas;
ViInt32 status;
vd = 1.5;
idcomp = 0.05;
vg = 1.5;
igcomp = 0.01; /* 29 */
ret = age5270_force(vi, bulk, age5270_VF_MODE, 0, 0, 0.1, 0); /* 31 */
ret = age5270_force(vi, source, age5270_VF_MODE, 0, 0, 0.1, 0);
ret = age5270_force(vi, gate, age5270_VF_MODE, 2, vg, igcomp, 0);
ret = age5270_force(vi, drain, age5270_VF_MODE, 2, vd, idcomp, 0);
check_err (vi, ret); /* 35 */
ret = age5270_spotMeas(vi,drain,age5270_IM_MODE,0,&meas,&status,0);
check_err (vi, ret); /* 38 */
ret = age5270_zeroOutput(vi, age5270_CH_ALL); /* 40 */
check_err (vi, ret); /* 41 */
printf("Id = %8.6f mA (at %2.1f V)\n", meas * 1000, vd); /* 43 */
printf("Vg = %2.1f V\n", vg);
ret = age5270_setSwitch(vi, age5270_CH_ALL, 0); /* 46 */
check_err (vi, ret); /* 47 */
}
 
ViStatus main ( )
{
 ViStatus    ret;
 ViSession   vi;
 ViChar      err_msg[256];

 /* Starting the session. */
 ret = age5270_init("GPIB::17::INSTR", VI_TRUE, VI_TRUE, &vi);
 if ( ( ret < VI_SUCCESS ) || ( vi == VI_NULL ) ) {
     printf("Initialization failure.\n Status code: %d.\n", ret);
     if ( vi != VI_NULL ) {
         age5270_error_message(vi, ret, err_msg);
         printf("Error: %ld\n %s\n", ret, err_msg);
	 }
     exit (ret);
 }
 ret = age5270_reset(vi);              	       /* Resets E5270              */
 ret = age5270_timeOut(vi, 60000); 	           /* sets 60 second timeout    */
 ret = age5270_errorQueryDetect(vi, VI_TRUE);  /* Turns on error detection. */
 
 perform_meas (vi, ret);
 /* ret = age5270_cmd(vi, "aa");
 check_err(vi, ret); */
 
 /* Closing the session. */
 ret = age5270_close(vi);
 check_err(vi, ret);

 return VI_SUCCESS;
}