#include   <stdio.h>
#include   <stdlib.h>
#include   <visa.h>
#include   "age5270.h"
 
void check_err (ViSession vi, ViStatus ret) {
 ViInt32    	inst_err;
 ViChar		    err_msg[256];
 if(VI_SUCCESS > ret) {
     if ( age5270_INSTR_ERROR_DETECTED == ret ) {
         age5270_error_query(vi, &inst_err, err_msg);
         printf("Instrument Error: %ld\n %s\n", inst_err, err_msg);
	 }
     else {
         age5270_error_message(vi, ret, err_msg);
         printf("Driver Error: %ld\n %s\n", ret, err_msg);
	 }
 }
}

void perform_meas (ViSession vi, ViStatus ret) /* 1 */
{
ViInt32 drain = 1; /* SMU1 */ /* 4 */
ViInt32 gate = 2; /* SMU2 */
ViInt32 source = 4; /* SMU4 */
ViInt32 bulk = 6; /* SMU6 */
ViReal64 vd = 3;
ViReal64 vg = 3;
ViReal64 idcomp = 0.05;
ViReal64 igcomp = 0.01;
ViReal64 hold = 0;
ViReal64 delay = 0;
ViReal64 s_delay = 0;
ViReal64 pdcomp = 0;
ViReal64 pgcomp = 0;
ViInt32 nop = 11;
ViInt32 rep;
ViReal64 sc[11];
ViReal64 md[11];
ViInt32 st[11];
ViReal64 tm[11];
ViInt32 i;
ViChar f_name[] = "data1.txt";
ViChar head1[] = "Vg (V), Id (mA), Time (sec), Status";
ViChar msg1[] = "Saving data...";
ViChar msg2[] = "Data save completed.";
ViChar c = '\n'; /* 31 */
ret = age5270_setSwitch(vi, drain, 1); /* 33 */
ret = age5270_setSwitch(vi, gate, 1);
ret = age5270_setSwitch(vi, source, 1);
ret = age5270_setSwitch(vi, bulk, 1);
check_err (vi, ret); /* 37 */
ret = age5270_resetTimestamp(vi); /* 39 */
ret = age5270_force(vi, bulk, age5270_VF_MODE, 0, 0, 0.1, 0);
ret = age5270_force(vi, source, age5270_VF_MODE, 0, 0, 0.1, 0);
ret = age5270_setIv(vi, gate, age5270_SWP_VF_SGLLIN, 0, 0, vg, nop, hold, delay, s_delay, igcomp, pgcomp);
check_err (vi, ret); /* 44 */
ret = age5270_setNthSweep(vi, 2, drain, age5270_VF_MODE, 0, 0, vd, idcomp, pdcomp);
check_err (vi, ret);
ret = age5270_msweepIv(vi, drain, age5270_IM_MODE, 0, &rep, &sc[0], &md[0], &st[0], &tm[0]);
check_err (vi, ret); /* 50 */
ret = age5270_zeroOutput(vi, age5270_CH_ALL); /* 52 */
ret = age5270_setSwitch(vi, age5270_CH_ALL, 0);
check_err (vi, ret); /* 54 */
if ( rep != nop ) { /* 56 */
printf ("%d measurement steps were returned.\nIt must be %d steps.\n", rep, nop);
exit (ret);
} /* 59 */
printf(" Vg (V), Id (mA)\n"); /* 61 */
for (i = 0; i < nop; i++){
printf(" %3.2f, %8.6f \n", sc[i], md[i] * 1000);
}
FILE *stream; /* 66 */
if( ( stream = fopen( f_name, "w+" )) == NULL ){
printf( "Data file was not opened\n" );
}
else {
printf( "%s%c", msg1, c );
fprintf( stream, "%s%c", head1, c );
for (i = 0; i < nop; i++){
fprintf( stream, "%3.2f, %8.6f, %8.6f, %d\n", sc[i], md[i] * 1000, tm[i], st[i]);
}
printf( "%s%c", msg2, c );
}
if( fclose( stream ) ){
printf( "Data file was not closed\n" );
} /* 82 */
}
 
ViStatus main ( )
{
 ViStatus    ret;
 ViSession   vi;
 ViChar      err_msg[256];

 /* Starting the session. */
 ret = age5270_init("GPIB::17::INSTR", VI_TRUE, VI_TRUE, &vi);
 if ( ( ret < VI_SUCCESS ) || ( vi == VI_NULL ) ) {
     printf("Initialization failure.\n Status code: %d.\n", ret);
     if ( vi != VI_NULL ) {
         age5270_error_message(vi, ret, err_msg);
         printf("Error: %ld\n %s\n", ret, err_msg);
	 }
     exit (ret);
 }
 ret = age5270_reset(vi);              	       /* Resets E5270              */
 ret = age5270_timeOut(vi, 60000); 	           /* sets 60 second timeout    */
 ret = age5270_errorQueryDetect(vi, VI_TRUE);  /* Turns on error detection. */
 
 perform_meas (vi, ret);
 ret = age5270_cmd(vi, "aa");
 check_err(vi, ret);
 
 /* Closing the session. */
 ret = age5270_close(vi);
 check_err(vi, ret);

 return VI_SUCCESS;
}