Imports Ivi.visa.interop

Module Module1


    Sub Main()
        Dim B1500 As IResourceManager
        Dim session As IMessage
        B1500 = New ResourceManager
        session = B1500.Open("GPIB0::17::INSTR")
        session.WriteString("*RST" & vbLf)
        MsgBox("Click OK to start measurement.", vbOKOnly, "")
        Console.WriteLine("Measurement in progress. . ." & Chr(10))

        Dim t() As Integer = {4, 3, 2, 1}
        Dim term As String = t(0) & "," & t(1) & "," & t(2) & "," & t(3)
        session.WriteString("CN " & term & vbLf)
        perform_meas(session, t)

        session.WriteString("CL" & vbLf)
        session.Close()
        MsgBox("Click OK to stop the program.", vbOKOnly, "")
        Console.WriteLine("Measurement completed." & Chr(10))
    End Sub

    Sub perform_meas(ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0                         't(0): Drain
        Dim j As Integer = 0                         't(1): Gate
        Dim nop1 As Integer = 1                      't(2): Source
        Dim nop2 As Integer = 1                      't(3): Substrate
        Dim data(nop2 - 1, nop1 - 1) As String
        Dim value As String = "Id (uA), Status"
        Dim fname As String = "C:\Agilent\prog_ex\data16.txt"
        Dim title As String = "Measurement Result"
        Dim msg As String = "No error."
        Dim err As Integer = 0

        Dim vd As Double = 3
        Dim vg As Double = 1
        Dim idcomp As Double = 0.05
        Dim igcomp As Double = 0.01
        Dim orng As Integer = 0
        Dim mrng As Integer = 0
        session.WriteString("FMT 3" & vbLf)
        session.WriteString("AV 10,1" & vbLf)     'sets number of samples for 1 data
        session.WriteString("FL 0" & vbLf)        'sets filter off
        session.WriteString("DV " & t(3) & ",0,0,0.1" & vbLf)   'out= 0 V, comp= 0.1 A
        session.WriteString("DV " & t(2) & ",0,0,0.1" & vbLf)   'out= 0 V, comp= 0.1 A
        session.WriteString("DV " & t(1) & "," & orng & "," & vg & "," & igcomp & vbLf)
        session.WriteString("DV " & t(0) & "," & orng & "," & vd & "," & idcomp & vbLf)

        session.WriteString("ERR? 1" & vbLf) : err = session.ReadString(4 + 2)
        If err <> 0 Then session.WriteString("DZ" & vbLf) : GoTo Check_err
        session.WriteString("TI " & t(0) & "," & mrng & vbLf)

        Dim dat() As Byte = session.Read(4 + 2) '4 byte data + terminator
        Dim status As Integer = dat(3) And 224 : status = status / 32 '224 = 128 + 64 + 32
        If status <> 0 Then session.WriteString("DZ" & vbLf) : GoTo Check_err
        Dim type As Integer = dat(0) And 128 : type = type / 128 '0:source, 1:meas
        Dim mode As Integer = dat(0) And 64 : mode = mode / 64 '0:voltage, 1:current
        Dim sign As Integer = dat(0) And 1 '0:positive, 1:negative
        Dim rng As Integer = dat(0) And 62 : rng = rng / 2
        Dim count As Integer = dat(1) * 256 + dat(2)
        Dim chan As Integer = dat(3) And 31 ' 31 = 16 + 8 + 4 + 2 + 1
        If sign = 1 Then count = count - 65536 '65536 = 10000000000000000 (17 bits)

        Dim range As Double
        If mode = 1 Then 'current range
            range = 10 ^ (rng - 20)
            If rng = 20 Then
                session.WriteString("UNT? 1" & vbLf)
                Dim unt As String = session.ReadString(256)
                Dim mdl(8) As String : Dim c As String
                Dim a As Integer : Dim b As Integer = 0 : Dim d As Integer = 0
                For a = 1 To Len(unt)
                    c = Mid(unt, a, 1)
                    If c = "," Then mdl(d) = Mid(unt, b + 1, a - b - 1) : d = d + 1
                    If c = ";" Then b = a
                Next
                If mdl(chan) = "E5291A" Then range = 0.2 'for E5260/E5270
            End If
        Else 'voltage range
            If rng = 8 Then range = 0.5
            If rng = 9 Then range = 5
            If rng = 11 Then range = 2
            If rng = 12 Then range = 20
            If rng = 13 Then range = 40
            If rng = 14 Then range = 100
            If rng = 15 Then range = 200
        End If

        value = value & Chr(13) & Chr(10) & "status = " & status
        value = value & Chr(13) & Chr(10) & "type = " & type
        value = value & Chr(13) & Chr(10) & "mode = " & mode
        value = value & Chr(13) & Chr(10) & "channel = " & chan
        value = value & Chr(13) & Chr(10) & "sign = " & sign
        value = value & Chr(13) & Chr(10) & "range = " & range
        value = value & Chr(13) & Chr(10) & "count = " & count & Chr(13) & Chr(10)

        Dim meas As Double
        If type = 0 Then meas = count * range / 20000 'source data
        If type = 1 Then meas = count * range / 50000 'measurement data

        data(j, i) = Chr(13) & Chr(10) & meas * 1000000 & ", " & status

        session.WriteString("DZ" & vbLf)
        save_data(fname, title, value, data, nop1, nop2, session, t)
        Exit Sub

Check_err:
        session.WriteString("EMG? " & err & vbLf) : msg = session.ReadString(256)
        MsgBox("Instrument error: " & err & Chr(10) & msg, vbOKOnly, "")
        Exit Sub

    End Sub


    Sub save_data(ByVal fname As String, ByVal title As String, ByVal value As String, ByVal data(,) As String, ByVal nop1 As Integer, ByVal nop2 As Integer, ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0
        Dim j As Integer = 0
        FileOpen(1, fname, OpenMode.Output, OpenAccess.Write, OpenShare.LockReadWrite)
        Print(1, value)
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                Print(1, data(j, i))
            Next
        Next
        FileClose(1)

        Dim rbx As Integer
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                value = value & data(j, i)
            Next
        Next
        value = value & Chr(10) & Chr(10) & "Data save completed."
        value = value & Chr(10) & Chr(10) & "Do you want to perform measurement again?"
        rbx = MsgBox(value, vbYesNo, title)
        If rbx = vbYes Then perform_meas(session, t)
    End Sub

End Module
