Imports Ivi.visa.interop

Module Module1


    Sub Main()
        Dim B1500 As IResourceManager
        Dim session As IMessage
        B1500 = New ResourceManager
        session = B1500.Open("GPIB0::17::INSTR")
        session.WriteString("*RST" & vbLf)
        MsgBox("Click OK to start measurement.", vbOKOnly, "")
        Console.WriteLine("Measurement in progress. . ." & Chr(10))

        Dim t() As Integer = {4, 3, 2, 1}
        Dim term As String = t(0) & "," & t(1) & "," & t(2) & "," & t(3)
        session.WriteString("CN " & term & vbLf)
        perform_meas(session, t)

        session.WriteString("CL" & vbLf)
        session.Close()
        MsgBox("Click OK to stop the program.", vbOKOnly, "")
        Console.WriteLine("Measurement completed." & Chr(10))
    End Sub

    Sub perform_meas(ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0                         't(0): Drain
        Dim j As Integer = 0                         't(1): Gate
        Dim nop1 As Integer = 11                     't(2): Source
        Dim nop2 As Integer = 1                      't(3): Substrate
        Dim data(nop2 - 1, nop1 - 1) As String
        Dim value As String = "Vg (V), Id (mA), Time (sec), Status"
        Dim fname As String = "C:\Agilent\prog_ex\data5.txt"
        Dim title As String = "Measurement Result"
        Dim msg As String = "No error."
        Dim err As Integer = 0

        Dim vd1 As Double = 0
        Dim vd2 As Double = 2
        Dim idcomp As Double = 0.05
        Dim pd_comp As Double = 0.1
        Dim vg1 As Double = vd1
        Dim vg2 As Double = vd2
        Dim igcomp As Double = 0.01
        Dim pg_comp As Double = 0.05
        Dim hold As Double = 0
        Dim delay As Double = 0
        Dim s_delay As Double = 0
        Dim rep As Integer = nop1

        Dim mret As String
        Dim sc(nop1) As Double
        Dim md(nop1) As Double
        Dim st(nop1) As String
        Dim tm(nop1) As Double
        session.Timeout = 60000
        session.WriteString("FMT 1,1" & vbLf)   'ASCII, <CRLF EOI>, w/sweep source data
        session.WriteString("TSC 1" & vbLf)     'enables time stamp output
        session.WriteString("FL 0" & vbLf)      'sets filter off
        session.WriteString("AV 10,1" & vbLf)   'sets number of samples for 1 data
        session.WriteString("DV " & t(3) & ",0,0,0.1" & vbLf)   'out= 0 V, comp= 0.1 A
        session.WriteString("DV " & t(2) & ",0,0,0.1" & vbLf)   'out= 0 V, comp= 0.1 A
        session.WriteString("MM 2," & t(0) & vbLf)              '2: staircase sweep measurement
        session.WriteString("CMM " & t(0) & ",1" & vbLf)        '1: current measurement
        session.WriteString("RI " & t(0) & ",0" & vbLf)         '0: auto ranging
        session.WriteString("WT " & hold & "," & delay & "," & s_delay & vbLf)
        session.WriteString("WM 2,1" & vbLf)                    'stops any abnormal
        session.WriteString("ERR? 1" & vbLf) : err = session.ReadString(4 + 2)
        If err <> 0 Then session.WriteString("DZ" & vbLf) : GoTo Check_err

        session.WriteString("WV " & t(0) & ",1,0," & vd1 & "," & vd2 & "," & nop1 & "," & idcomp & "," & pd_comp & vbLf)
        session.WriteString("WSV " & t(1) & ",0," & vg1 & "," & vg2 & "," & igcomp & "," & pg_comp & vbLf)
        session.WriteString("TSR" & vbLf)
        session.WriteString("XE" & vbLf)
        session.WriteString("*OPC?" & vbLf) : rep = session.ReadString(1 + 2)
        session.WriteString("ERR? 1" & vbLf) : err = session.ReadString(4 + 2)
        If err <> 0 Then session.WriteString("DZ" & vbLf) : GoTo Check_err
        session.WriteString("NUB?" & vbLf) : rep = session.ReadString(3 + 2)
        If rep <> nop1 * 3 Then session.WriteString("DZ" & vbLf) : GoTo Check_nop

        mret = session.ReadString(16 * 3 * nop1 + 1)
        For i = 0 To nop1 - 1
            tm(i) = Val(Mid(mret, 4 + 16 * 3 * i, 12))
            st(i) = Mid(mret, 17 + 16 * 3 * i, 3)
            md(i) = Val(Mid(mret, 20 + 16 * 3 * i, 12))
            sc(i) = Val(Mid(mret, 36 + 16 * 3 * i, 12))
            data(j, i) = Chr(13) & Chr(10) & sc(i) & ", " & md(i) * 1000 & ", " & tm(i) & ", " & st(i)
        Next i

        session.WriteString("DZ" & vbLf)
        save_data(fname, title, value, data, nop1, nop2, session, t)
        Exit Sub

Check_err:
        session.WriteString("EMG? " & err & vbLf) : msg = session.ReadString(256)
        MsgBox("Instrument error: " & err & Chr(10) & msg, vbOKOnly, "")
        Exit Sub

Check_nop:
        MsgBox("No. of data: " & rep & " (not " & nop1 * 3 & ")", vbOKOnly, "")
    End Sub

    Sub save_data(ByVal fname As String, ByVal title As String, ByVal value As String, ByVal data(,) As String, ByVal nop1 As Integer, ByVal nop2 As Integer, ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0
        Dim j As Integer = 0
        FileOpen(1, fname, OpenMode.Output, OpenAccess.Write, OpenShare.LockReadWrite)
        Print(1, value)
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                Print(1, data(j, i))
            Next
        Next
        FileClose(1)

        Dim rbx As Integer
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                value = value & data(j, i)
            Next
        Next
        value = value & Chr(10) & Chr(10) & "Data save completed."
        value = value & Chr(10) & Chr(10) & "Do you want to perform measurement again?"
        rbx = MsgBox(value, vbYesNo, title)
        If rbx = vbYes Then perform_meas(session, t)
    End Sub

End Module
